/*
** cmdproc.cpp --- subroutines respect to each command.
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2001/01/25 17:37:18 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <string>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include "dic.h"
#include "spcsrv.h"
#include "ssqueue.h"
#include "kanjifn.h"
#include "cmdproc.h"

int
cmd_q(ss_queue *q, char *pcCmd, char **pcEndCmd)
{
  char *p = pcCmd;
  char *str_begin;
  int iRet;
  int adding;

  if (*p != 'q')
    adding = 1;
  else {
    adding = 0;
    while (*p != '{' && *p != '\0')
      p++;
  }
  if (*p == '\0') {
    *pcEndCmd = p;
    return 1;
  }

  if (*p == '{') p++;
  str_begin = p;

  while (*p != '}' && *p != '\0') {
    if (iskanji(*p)) p+=2;
    else p++;
  }
  // if the string is not empty
  if (p != str_begin) {
    if (adding) {
      q->push_text_add(str_begin, (p - str_begin));
    } else {
      q->push_text(str_begin, (p - str_begin));
    }
  }
  if (*p == '}') {
    *pcEndCmd = p+1;
    iRet = 0;
  } else {
    *pcEndCmd = p;
    iRet = 1;
  }
  return iRet;
}

int
cmd_d(ss_queue *q, dTalkParam *dt,
      ssSharedInfo *info, char *pcCmd, char **pcEndCmd)
{
  if (*pcCmd == '\0') {
    *pcEndCmd = pcCmd;
    return 0;
  }

  while(!q->empty()) {
    q->pop(dt, info);
  }

  *pcEndCmd = pcCmd+1;
  return 0;
}

void
cmd_s(ss_queue *q, ssSharedInfo *info, char *pcCmd, char **pcEndCmd)
{
  if (*pcCmd == '\0') {
    *pcEndCmd = pcCmd;
    return;
  }
  printf("stopping\n");
  dt_stop(info);
  q->clear();

  *pcEndCmd = pcCmd+1;
  return;
}

int
cmd_unknown(char *pcCmd, char **pcEndCmd)
{
  char *p = pcCmd;
  int iRet;
  if (*p == '\0') {
    *pcEndCmd = p;
    return 0;
  }
  // skip command
  while (*p != '\0' && *p != ';' && *p != '{')
    p++;

  // skip argument if any
  if(*p == '{') {
    while (*p != '}' && *p != '\0') {
      if (iskanji(*p)) p+=2;
	  else p++;
    }
      if (*p == '}') {
	*pcEndCmd = p+1;
	iRet = 0;
      } else {
	*pcEndCmd = p;
	iRet = 1;
      }
  }
  else {
    while (*p != '\0' && (*p == ';' || *p == ' '))
	   p++;
    *pcEndCmd = p;
    iRet = 0;
  }

  return iRet;
}

void
cmd_tts_set_speech_rate(VoiceSet *vset, char *pcCmd, char **pcEndCmd)
{
  int speed;
  if (*pcCmd == '\0') {
    *pcEndCmd = pcCmd;
    return;
  }

  while(*pcCmd != ' ' && *pcCmd != '\0')
    pcCmd++;
  if (*pcCmd == '\0') {
    *pcEndCmd = pcCmd;
    return;
  }
  while (*pcCmd == ' ')
    pcCmd++;
  speed = atoi(pcCmd);
  vset->iSpeed = speed;
  while (*pcCmd >= '0' && *pcCmd <= '9')
    pcCmd++;
  *pcEndCmd = pcCmd;
  return;
}

void
cmd_tts_set_punctuations(VoiceSet *vset, char *pcCmd, char **pcEndCmd)
{
  if (*pcCmd == '\0') {
    *pcEndCmd = pcCmd;
    return;
  }

  while(*pcCmd != ' ' && *pcCmd != '\0')
    pcCmd++;
  if (*pcCmd == '\0') {
    *pcEndCmd = pcCmd;
    return;
  }
  while (*pcCmd == ' ')
    pcCmd++;
  if (strncmp("all", pcCmd, 3)==0)
    vset->iPunctMode = SS_PUNCTUATION_ALL;
  if (strncmp("some", pcCmd, 4)==0)
    vset->iPunctMode = SS_PUNCTUATION_SOME;
  if (strncmp("none", pcCmd, 4) == 0)
    vset->iPunctMode = SS_PUNCTUATION_NONE;
  while (*pcCmd >= '0' && *pcCmd <= '9')
    pcCmd++;
  *pcEndCmd = pcCmd;
  return;
}

int
cmd_tts_say(ss_queue *q, dTalkParam *dt,
	    ssSharedInfo *info, char *pcCmd, char **pcEndCmd)
{
  char *p = pcCmd;
  char *str_begin;
  int iRet;

  while (*p != '{' && *p != '\0')
    p++;
  if (*p == '\0') {
    *pcEndCmd = p;
    return 0;
  }

  if (*p == '{') p++;
  str_begin = p;

  int iteration = 0;
  while (*p != '}' && *p != '\0') {
    while (*p != '}' && *p != '\0') {
      if (iskanji(*p)) p+=2;
      else p++;
    }
    // if the string is not empty
    if (p != str_begin) {
      if (iteration) {
	q->push_text_add(str_begin, (p - str_begin));
      } else {
	q->push_text(str_begin, (p - str_begin));
      }
    }
    str_begin = p;
    iteration++;
  }
  if (*p == '}') {
    *pcEndCmd = p+1;
    iRet = 0;
  } else {
    *pcEndCmd = p;
    iRet = 1;
  }
  q->pop(dt, info);
  return iRet;
}

void
cmd_l(ss_queue *q, dTalkParam *dt,
      ssSharedInfo *info, char *pcCmd, char **pcEndCmd)
{
  char *p = pcCmd;
  char *str_begin;
  string text = "";
  pthread_t th;

  dt_stop(info);

  while (*p != '{' && *p != '\0')
    p++;
  if (*p == '\0') {
    *pcEndCmd = p;
    return;
  }

  if (*p == '{') p++;
  str_begin = p;

  int iteration = 0;
  while (*p != '}' && *p != '\0') {
    while (*p != '}' && *p != '\0') {
      if (iskanji(*p)) p+=2;
      else p++;
    }
    // if the string is not empty
    if (p != str_begin) {
      if (iteration) {
	text.append(str_begin, (p - str_begin));
      } else {
	text.assign(str_begin, (p - str_begin));
      }
    }
    str_begin = p;
    iteration++;
  }
  ssWorkArg *wArg = new ssWorkArg();
  wArg->param = dt;
  wArg->vset = q->vset;
  wArg->info = info;
  strcpy(wArg->src, text.c_str());
  wArg->iLen = text.length();
  wArg->iSpeed = q->vset->iSpeed/dt->iSpeedFactor;
  if(wArg->iSpeed > DT_MAX_SPEED) {
    wArg->iSpeed = DT_MAX_SPEED;
  }
  wArg->type = SS_WORK_TYPE_PLAIN;
  pthread_create(&th, NULL, dt_speak, (void *)wArg);
  //  pthread_detach(th);
  if (*p == '}') {
    *pcEndCmd = p+1;
  } else {
    *pcEndCmd = p;
  }
  return;
}
