/*
** dtfunc.cpp --- Voice synthesis functions.
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2001/07/21 08:31:10 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <string>
#include <time.h>
#include "dic.h"
#include "kanjifn.h"
#include "spcsrv.h"
#if SS_SYN_USE_ESD==TRUE
#include <esd.h>
#endif
VoiceSet::VoiceSet()
{
  voiceNum = 10;
  voices = new Voice[voiceNum];

  voices[0].name = "paul";
  voices[0].init_string = "M3I2 ";

  voices[1].name = "betty";				// use mary for betty
  voices[1].init_string = "F3I2 ";

  voices[2].name = "paul-monotone";
  voices[2].init_string = "M3I0 ";

  voices[3].name = "paul-animated";
  voices[3].init_string = "M4I3 ";

  voices[4].name = "paul-smooth";
  voices[4].init_string = "M3I2 ";

  voices[5].name = "indent-voice";
  voices[5].init_string = "M1I2 ";

  voices[6].name = "harry";
  voices[6].init_string = "M2I2 ";

  voices[7].name = "ursula";
  voices[7].init_string = "F2I2 ";

  voices[8].name = "robosoft3";
  voices[8].init_string = "M3I1 ";

  voices[9].name = "robosoft4";
  voices[9].init_string = "F3I1 ";
  pthread_mutex_init(&lock, NULL);
  currentVoice = 0;
  iSpeed = SS_INIT_SPEED;
  iPunctMode = SS_PUNCTUATION_SOME;
}

VoiceSet::~VoiceSet()
{
  delete [] voices;
  pthread_mutex_destroy(&lock);
}

int
VoiceSet::selectVoice(const char *name)
{
  int i;
  pthread_mutex_lock(&lock);
  for (i=0; i < voiceNum; i++) {
    if (voices[i].name == name) {
      break;
    }
  }
  if (i == voiceNum) {
    pthread_mutex_unlock(&lock);
    return false;
  }
  else {
    currentVoice = i;
  }
  pthread_mutex_unlock(&lock);
  return true;
}

string &
VoiceSet::initString()
{
  return voices[currentVoice].init_string;
}
dTalkParam::dTalkParam()
{
  pthread_mutex_init(&lock, NULL);
}

dTalkParam::~dTalkParam()
{
  pthread_mutex_destroy(&lock);
}

int
dTalkParam::setSpeed(int speed)
{
  int iRet;
  pthread_mutex_lock(&lock);
  iRet = SYT_setSpeed ( SynHandle,
		       speed);
  pthread_mutex_unlock(&lock);
  return iRet;
}

int
dTalkParam::setPunctuation(int mode)
{
  int dtMode, iRet;
  pthread_mutex_lock(&lock);
  switch (mode) {
  case SS_PUNCTUATION_ALL:
  case SS_PUNCTUATION_SOME:
    dtMode = LG_SIGN_ON;
    break;
  case SS_PUNCTUATION_NONE:
    dtMode = LG_SIGN_OFF;
    break;
  default:
    dtMode = LG_SIGN_OFF;
  }
  iRet = LNG_setSign ( tLngHandle,
		       dtMode);
  pthread_mutex_unlock(&lock);
  return iRet;
}

ssSharedInfo::ssSharedInfo()
{
#if 0
  pthread_mutexattr_settype(&lockattr, 
			       PTHREAD_MUTEX_RECURSIVE_NP);
#endif
  pthread_mutex_init(&lock, NULL);
  flag = SS_FLAG_UNSET;
}

ssSharedInfo::~ssSharedInfo()
{
  pthread_mutex_destroy(&lock);
}

void
ssInitWorkArg(ssWorkArg *wArg)
{
  wArg->iSpeed = SS_INIT_SPEED;
  wArg->type = SS_WORK_TYPE_PLAIN;
  wArg->ret = 0;
}

int dt_init(dTalkParam *dt)
{
  int iRet;
  int iCount;
  int iErrorInfo;
  int failed;

  dt->pszSynDic  = SS_DTALKER_DIC_DIR "/" SS_DTALKER_SYN_DIC;
  strcpy(dt->szLangDic, SS_DTALKER_DIC_DIR "/" SS_DTALKER_LNG_DIC);
#if 0
  strcpy(dt->szUsrDic1, SS_LNG_DIC_DIRECTORY "/" SS_LNG_DIC_ENG_1);
  strcpy(dt->szUsrDic2, SS_LNG_DIC_DIRECTORY "/" SS_LNG_DIC_ENG_2);
#endif
#if SS_SYN_USE_ESD==TRUE
  dt->pszDsp = SS_SYN_ESD_HOST;
#else
  dt->pszDsp = SS_SYN_OUTPUT_DEVICE;
#endif
  dt->SynHandle = NULL;
  dt->iSpeedFactor = DT_INIT_SPEED_FACTOR;

  /* load English to Kana dictionary */
  iRet = dicLoadDictionary(&(dt->engKanaDic),
			   SS_DTALKER_DIC_DIR "/" SS_ENG_KANA_DIC);
  if (iRet != 0) {
    fprintf(stderr, "can not load Eng to Kana dictionary\n");
    return SS_OPEN_FAIL;
  }

  failed = 0;

  /*
   * Υץ
   */
  dt->tLngOpen.pszSysDic = dt->szLangDic;			/* ܸ켭ե̾ */
  for ( iCount = 0; iCount < 15; iCount++ )
    dt->tLngOpen.pszUserDic[ iCount ] = NULL;	/* 桼켭ϻѤʤ */
#if 0
  dt->tLngOpen.pszUserDic[0] = dt->szUsrDic1;
  dt->tLngOpen.pszUserDic[1] = dt->szUsrDic2;
#endif
  dt->tLngOpen.wCharSet = LG_CODE_MS;	/* ޥեȥɷ */
  dt->tLngOpen.wState = 0;	/* ơ򥯥ꥢ */

  iRet = LNG_analyzeInit ( &dt->tLngHandle, &dt->tLngOpen );
  if ( iRet != LGERR_NOERR )
    {
      fprintf (stderr, "LNG_analyzeInit failed, code %d\n", iRet);
      return SS_OPEN_FAIL;;
    }

  /*
   * ȷΥץ
   */
  dt->SynHandle = SYT_syntheInit ( dt->pszSynDic, 
				   STY_WAVE_KIND_BOTH, 
				   STY_WAVE_DATA_16LINEAR, 
				   /*STY_WAVE_DATA_8MLAW,*/
				   &iErrorInfo, 
				   "DT_TTS_mmf" );
  if ( dt->SynHandle == NULL )
    {
      fprintf (stderr, "SYT_syntheInit failed\n");
      dt_close(dt);
      return SS_OPEN_FAIL;
    }

  return SS_OPEN_OK;
}

int dt_close(dTalkParam *dt)
{
  int iRet;
  int iErrorInfo;
  int failed = 0;

  /*
   * Υ
   */
  iRet = LNG_analyzeEnd ( dt->tLngHandle );
  if ( iRet != LGERR_NOERR )
    {
      fprintf (stderr, "LNG_analyzeInit failed\n");
      failed++;
    }

  /*
   * ȷΥ
   */
  if (dt->SynHandle != NULL) {
    iErrorInfo = SYT_syntheEnd ( dt->SynHandle );
    if ( iErrorInfo != SYERR_NOERROR )
      {
	fprintf (stderr, "SYT_syntheEnd failed\n");
	failed++;
      }
  }

  if (failed) {
    return SS_CLOSE_FAIL;
  }
  return SS_CLOSE_OK;
}

int dt_lang_anal(dTalkParam *dt, char *src)
{
  int iRet;

  pthread_mutex_lock(&dt->lock);
  /*
   * Ϥμ¹
   */
  dt->tLng.pszSrc = src;			/*  оʸ */
  dt->tLng.pszDst = NULL;				/*   */
  dt->tLng.wCnvMode = CNV_FIRST | CNV_LAST;		/* Ѵ⡼ */
  dt->tLng.wReserved = 0;					/* ꥶ */
  dt->tLng.dwDstLength = 0;				/* Ϸ̤Ĺ򥯥ꥢ */

  iRet = LNG_analyze( dt->tLngHandle, &dt->tLng );
  pthread_mutex_unlock(&dt->lock);
  if ( iRet != LGERR_NOERR )
    {
      fprintf (stderr, "LNG_analyze failed\n");
      return SS_LANG_ANAL_FAIL;
    }
  
  return SS_LANG_ANAL_OK;
}

void
doBracketCommand(ssWorkArg *arg, const char *cmd)
{
  char *p;
  if (cmd[0] != ':')
    return;
#if DEBUG >= 3
  printf("processing command [%s]\n", cmd);
#endif
  if (strncmp(":name", cmd, 5) == 0) {
    p = (char *)cmd+5;
    while(*p == ' ') p++;
    arg->vset->selectVoice(p);
    return;
  }
  if (strncmp(":n", cmd, 2) == 0) {
    switch (cmd[2]) {
    case 'p':
      if (strncmp("p-monotone", &cmd[2], 10)==0)
	arg->vset->selectVoice("paul-monotone");
      else if (strncmp("p-animated", &cmd[2], 10) == 0)
	arg->vset->selectVoice("paul-animated");
      else if (strncmp("p-smooth", &cmd[2], 8) == 0)
	arg->vset->selectVoice("paul-smooth");
      else arg->vset->selectVoice("paul");
      break;
    case  'b':
      arg->vset->selectVoice("betty");
      break;
    case 'i':
      arg->vset->selectVoice("indent-voice");
    case 'h':
      arg->vset->selectVoice("harry");
    case 'u':
      arg->vset->selectVoice("ursula");
    }
  }
  return;
}

void *
dt_speak(void *wArg)
{
  int iRet;
  int iErrorInfo;  /* DTalkerΥ顼 */
  unsigned long dwGenSize;	 /* ºݤ줿PCMǡΥ */
  char pcPcmBuf[SS_PCMSIZE];
  char *pcEngKanaResult;
  int fd_audio = -1;    /* ѥեǥץ */
#if SS_SYN_USE_ESD==TRUE
  char *host = SS_SYN_ESD_HOST;
  char prog_name[SS_MAX_PROG_NAME_LEN];
#endif
  ssWorkArg arg_obj = *((ssWorkArg *)wArg);
  ssWorkArg *arg = &arg_obj;
  dTalkParam *dt = arg->param;
  ssSharedInfo *info = arg->info;
  char *pcBeg, *pcEnd, *pcWork, *pcWork2;
  int engFlag, engFlagTmp;
  int iLen;
  string tmpStr, phonStr;

  delete wArg;

  /* stop if flag is set */
  if (info->flag & SS_FLAG_REQUEST_STOP) {
    arg->vset->selectVoice("paul");
    goto EXIT_OK;
  }
  /* wait for preceeding work */
#if DEBUG >= 3
  printf("dt_speak input: %s\n", arg->src);
#endif

  /* initialize pointer */
  pcBeg = arg->src;

  iLen = strlen(arg->src);

  pthread_mutex_lock(&info->lock);
  while(1) {
    /* eat spaces */
    while(*pcBeg == ' ' && *pcBeg != '\0')
      pcBeg++;
#if DEBUG >= 3
    printf("remaining: %s\n", pcBeg);
#endif
    pcEnd = pcBeg;

    /* if the end of src */
    if (*pcBeg == '\0') {
      break;
    }

    /* process [] command */
    if (*pcBeg == '[') {
      while (*pcEnd != ']' && *pcEnd != '\0')
	pcEnd++;
      *pcEnd = '\0';
      doBracketCommand(arg, pcBeg+1);
      pcBeg = pcEnd;
      if (pcBeg - arg->src >= iLen)
	break;
      pcBeg++;
      continue;
    }

    /* stop if flag is set */
    if (info->flag & SS_FLAG_REQUEST_STOP) {
      arg->vset->selectVoice("paul");
      goto EXIT_OK;
    }

    /* for speakable text */
    while (*pcEnd != '[' && *pcEnd != '\0'
	   && pcEnd - arg->src < iLen) {
      if (iskanji(*pcEnd)) pcEnd+=2;
      else pcEnd++;
    }

    /* check English word and convert to kana */
    tmpStr = "";
    pcWork = pcBeg;
    pcWork2 = pcWork;
    engFlag = FALSE;
    while (pcWork <= pcEnd) {
      if ((pcWork != pcEnd)
	  && (iszenalpha((unsigned char *)pcWork) || isalnum(*pcWork)
	      || *pcWork == '\''))
	engFlagTmp = TRUE;
      else engFlagTmp = FALSE;
      if (pcBeg == pcWork) { /* if pcWork points the first char */
	engFlag = engFlagTmp;
	if (iskanji(*pcWork)) pcWork+=2;
	else pcWork++;
	continue;
      }
      if (engFlag != engFlagTmp || pcWork == pcEnd) {
	if (engFlag == TRUE
	    &&jpEngToKana(&(dt->engKanaDic), pcWork2,
			  (pcWork-pcWork2), &pcEngKanaResult)) {
#if DEBUG >= 3
	  printf("converted: %s\n", pcEngKanaResult);
#endif
	  tmpStr.append(pcEngKanaResult);
	  free (pcEngKanaResult);
	}
	else {
	  if ((pcWork2 != pcBeg && !isdigit(*(pcWork2-1)))
	      && *pcWork2 == ' ') pcWork2++;
	  tmpStr.append(pcWork2, (pcWork-pcWork2));
	}
	pcWork2 = pcWork;
	engFlag = engFlagTmp;
      }
      if (pcWork == pcEnd) break;
      /* goto next char */
      if (iskanji(*pcWork)) pcWork+=2;
      else pcWork++;
    }

#if DEBUG >= 3
    printf("speaking: %s\n", tmpStr.c_str());
#endif
    pcBeg = pcEnd;

    /* set various modes */
    iErrorInfo = dt->setSpeed (arg->iSpeed);
    if (iErrorInfo != SYERR_NOERROR)
      {
	fprintf(stderr, "SYT_setSpeed() failed\n");
	goto EXIT_FAIL;
      }

    iErrorInfo = dt->setPunctuation (arg->vset->iPunctMode);
    if (iErrorInfo != SYERR_NOERROR)
      {
	fprintf(stderr, "SYT_setSpeed() failed\n");
	goto EXIT_FAIL;
      }

    /* language analysis */
    iRet = dt_lang_anal(dt, (char *)tmpStr.c_str());
    if (iRet != SS_LANG_ANAL_OK) {
      goto EXIT_FAIL;
    }
    if (dt->tLng.pszDst == NULL) {
      continue; /* silently ignore */
    }
  
    /* ѥǥǥХΥץ */
    fd_audio = -1;
#if SS_SYN_USE_ESD == TRUE
    fd_audio = open_audio(host, prog_name); /* output device */
#else
    fd_audio = open_audio(dt->pszDsp); /* output device */
#endif
    if ( fd_audio == -1 )
      {
	fprintf (stderr, "cannot open audio device %s\n",
		 dt->pszDsp );
	goto EXIT_FAIL;
      }

    /* prefix phonetic data with initString */
    phonStr = arg->vset->initString();
    phonStr += dt->tLng.pszDst;

    /*
     * ɽʸPCMǡѴ
     */

#if DEBUG >= 3
    printf("Synthesis: %s\n", phonStr.c_str());
#endif
    iErrorInfo = SYT_generatePcmStart ( dt->SynHandle,
					phonStr.c_str(),
					0);
    if ( iErrorInfo != SYERR_NOERROR )
      {
	fprintf (stderr, "SYT_generatePcmStart failed, %d\n",
		 iErrorInfo);
	if (!ss_close (fd_audio)) {
	  fd_audio = -1;
	}
	goto EXIT_FAIL;
      }

    for ( ; ; ) {
      /* check stop request */
      if (info->flag & SS_FLAG_REQUEST_STOP) {
	SYT_generatePcmStop(dt->SynHandle);
#if DEBUG >= 3
	printf("stop request accepted\n");
#endif
#if SS_SYN_USE_ESD==TRUE
#if DEBUG >= 3
	printf("reset\n");
#endif
	ss_reset_audio(prog_name, host);
#else
	ss_reset_audio(fd_audio);
	write (fd_audio, pcPcmBuf, 0);
#endif
	if(!ss_close (fd_audio)) {
	  fd_audio = -1;
	}
	arg->vset->selectVoice("paul");
	break;  /* out of for loop */
      }
      iErrorInfo = SYT_generatePcm ( dt->SynHandle, 
				     pcPcmBuf,
				     SS_PCMSIZE, 
				     &dwGenSize );
      if ( iErrorInfo == SYRET_CONTINUE || iErrorInfo == SYRET_END )
	{
	  if (iErrorInfo == SYRET_CONTINUE) {
	    int size;
	    size = ssWarpPcm((short *)pcPcmBuf, dwGenSize/2, 0.95);
	    /* PCMǡ򥪡ǥǥХ˽񤭹 */
	    if ( write ( fd_audio, pcPcmBuf,
			 size*2 ) != size*2 )
	      {
		fprintf (stderr, "cannot write to audio\n");
		goto EXIT_FAIL;
	      }
	  }
	  /* ⤦PCMǡϻĤäƤʤ */
	  if ( iErrorInfo == SYRET_END ) {
	    write (fd_audio, pcPcmBuf, 0);
	    if(!ss_close (fd_audio)) {
	      fd_audio = -1;
	    }
	    break;
	  }
	}
      else
	{
	  /* PCMǡ˼Ԥ */
	  fprintf (stderr, "SYT_generatePcm failed\n");
	  goto EXIT_FAIL;
	}
#if SS_SYN_USE_ESD == TRUE
	    {
	      struct timespec tm;
	      tm.tv_sec = 0;
	      tm.tv_nsec = 155000000;
	      nanosleep(&tm, NULL);
	    }	    
#endif
    }
  }
#if DEBUG >= 3
  printf("output complete\n");
#endif

  if ((fd_audio != -1) && ss_close ( fd_audio ) == -1 )
    {
      fprintf (stderr, "audio cannot close %s\n", dt->pszDsp );
      goto EXIT_FAIL;
    }

 EXIT_OK:
  pthread_mutex_unlock(&info->lock);
  return (void *)SS_SPEAK_OK;
 EXIT_FAIL:
  if (fd_audio != -1) {
    if(!ss_close (fd_audio)) {
      fd_audio = -1;
    }
  }
  pthread_mutex_unlock(&info->lock);
  return (void *)SS_SPEAK_FAIL;
}

void dt_stop(ssSharedInfo *info)
{
  if (info->flag & SS_FLAG_REQUEST_STOP)
    return;
  info->flag |= SS_FLAG_REQUEST_STOP;
  pthread_mutex_lock(&info->lock);
  info->flag = SS_FLAG_UNSET;
  pthread_mutex_unlock(&info->lock);
#if SS_SYN_USE_ESD == TRUE
  /* wait a little */
  {
    struct timespec tm;
    tm.tv_sec = 0;
    tm.tv_nsec = 100000000;
    nanosleep(&tm, NULL);
  }	    
#endif
  return;
}
