/*
** kanjifn.cpp --- Kanji-related functions.
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2001/01/14 08:42:09 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "dic.h"
#include "spcsrv.h"
#include "kanjifn.h"

/* convert Japanese jisx0208 alphabet and numeric to ascii */
char
jpZenToHan(char *pc)
{
  unsigned char c1 = (unsigned char)(*pc);
  unsigned char c2 = (unsigned char)(*(pc+1));
  if (c1 == 0x82 && c2 == 0x66)
    return '\'';
  if (c1 != 0x82)
    return '?';
  if (c2 >= 0x4f && c2 <= 0x58)
    return (c2 - 0x4f + '0');
  if (c2 >= 0x60 && c2 <= 0x79)
    return (c2 - 0x60 + 'A');
  if (c2 >= 0x81 && c2 <= 0x9a)
    return (c2 - 0x81 + 'a');
  return '?';
}

int
jpEngToKana(Dictionary *dicp, char *pszIn, int inLen, char **ppszOut)
{
  char *buf = NULL; /* normalized alphabet buffer */
  dicRecord *recBuf[ENG_KANA_MAX_FLAGMENT];
  dicRecord *pdr;
  dicRecord drWordTail;
  int i, matchFlag;
  int iWords, iBeg, iEnd, iEndTmp;
  char *p, *p2;
  /* list of word tails */
  char *tails[] = {"S", "Y", "ES", "CY", "\'S", "Y", "S\'", "Y\'",
		   "D", "hD", "ED", "hD", "ING", "CO",
		   "LY", "", NULL};
  *ppszOut = NULL;
  printf("matching ");
  for (int iLoop=0; iLoop < inLen; iLoop++)
    printf("%c", pszIn[iLoop]);
  /* Allocate buf. It is always shorter than inLen */
  buf = (char *)malloc(sizeof(char)*inLen+1);
  if (buf == NULL) {
    *ppszOut = NULL;
    return FALSE;
  }
  /* copy to buf capitalizing and hankaku-converting */
  p = pszIn;
  p2 = buf;
  while (p - pszIn < inLen) {
    if (iszenalpha((unsigned char *)p)) {
      *(p2++) = toupper(jpZenToHan(p));
      p += 2;
    }
    else {
      *p2++ = toupper(*p++);
    }
  }
  *p2 = '\0';
  printf(" normalized: %s\n", buf);
  /* find the first matching word conbination */
  matchFlag = 1;
  iWords = 0;
  iBeg = 0;
  iEnd = strlen(buf);
  while (iBeg < iEnd) {
    /* check the rule of word tails */
    if (iBeg > 0) {
      for (i = 0; tails[i] != NULL; i+=2) {
	if (strcmp(tails[i], &buf[iBeg]) != 0)
	  continue;
	drWordTail.key = tails[i];
	drWordTail.cont = tails[i+1];
	drWordTail.attr = 0;
	drWordTail.next = NULL;
	recBuf[iWords++] = &drWordTail;
	printf("%s\n", pdr->key);
	iBeg = iEnd;
	break;
      }
      if (tails[i] != NULL)
	continue;
    }
    for (iEndTmp = iEnd; iEndTmp > iBeg+1; iEndTmp--) {
      pdr = dicNHashSearch(dicp, &buf[iBeg], iEndTmp-iBeg);
      if (pdr == NULL) continue; /* not found */
      /* If attribute is 1, it is a match only when it is the
	 entire word. */
      if (pdr->attr == 1 && !(iBeg == 0 && iEndTmp == iEnd)) {
	continue;
      }
      /* match found */
      recBuf[iWords++] = pdr;
      printf("%s\n", pdr->key);
      iBeg = iEndTmp;
      break;
    }
    /* If no match is found in this loop, we don't know this word. */
    if (iEndTmp == iBeg+1) {
      matchFlag = 0;
      break;
    }
  }
  /* buf is not needed anymore */
  free(buf);
  if (matchFlag == 0) {
    return FALSE;
  }
  /* count length of converted kana */
  for (i = 0, iEndTmp = 0; i < iWords; i++) {
    iEndTmp += strlen(recBuf[i]->cont);
  }
  /* allocate new buffer for output */
  buf = (char *)malloc(sizeof(char) * iEndTmp);
  if (buf == NULL) {
    return FALSE;
  }
  /* copy kana string */
  buf[0] = '\0';
  for (i=0; i < iWords; i++) {
    printf("%s\n", recBuf[i]->cont);
    strcat(buf, recBuf[i]->cont);
  }
  *ppszOut = buf;
  return TRUE;
}
