/*
** spcsrv.h --- Various declarations for LinuxSpeechServer
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2001/01/25 17:37:18 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef __SPCSRV_H__
#define __SPCSRV_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string>
#include <dtalker/synthe.h>
#include <dtalker/lgproc.h>
#include "config.h"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

/* input buffer */
#define SS_INPUT_BLK_SIZE 512
/* PCM buffer size */
#define SS_PCMSIZE 6144
/* initial speed */
#define SS_INIT_SPEED 300
#define DT_INIT_SPEED_FACTOR 30
#define DT_INIT_SPEED 9
#define DT_MAX_SPEED 9
#define SS_MAX_PROG_NAME_LEN 288
#define SS_WORK_STR_MAX 16384
#define SS_WORK_TYPE_PLAIN 1  /* $BF~NO$ODL>o$N%F%-%9%H(B */
#define SS_WORK_TYPE_PHONETIC 2  /* $BF~NO$O%(%s%8%sMQ%U%)%M%F%#%C%/!#8@8l=hM}ITMW(B */
#define SS_OPEN_OK 0
#define SS_OPEN_FAIL 1
#define SS_CLOSE_OK 0
#define SS_CLOSE_FAIL 1
#define SS_LANG_ANAL_OK 0
#define SS_LANG_ANAL_FAIL 1
#define SS_SPEAK_OK 0
#define SS_SPEAK_FAIL 1
#define SS_FLAG_UNSET 0
#define SS_FLAG_REQUEST_STOP 0x01
#define SS_PUNCTUATION_ALL 0
#define SS_PUNCTUATION_SOME 1
#define SS_PUNCTUATION_NONE 2

struct Voice {	// base class for voices to derive from
  string name;
  string init_string;	// to set the voice mode
};

struct VoiceSet {
  Voice *voices;
  int voiceNum;
  int currentVoice;
  int iSpeed;
  int iPunctMode;
  pthread_mutex_t lock;
  VoiceSet();
  ~VoiceSet();
  int selectVoice(const char *);
  string &initString();
};

/* Shared Information */
struct ssSharedInfo {
  long flag;
  pthread_mutex_t lock;
  ssSharedInfo();
  ~ssSharedInfo();
};

struct dTalkParam {
  Dictionary engKanaDic;
  char *pszSynDic;						/* $BGH7A<-=q%U%!%$%kL>(B */
  char szLangDic[200];			/* $B4pK\8@8l<-=q%U%!%$%kL>(B */
  char szUsrDic1[200];
  char szUsrDic2[200];
  char *pszDsp;                    /* $B=PNO%*!<%G%#%*%G%P%$%9(B */
  int iSpeedFactor;  /* $BA4BN$NB.EYCM$r$3$l$G3d$k!#(B */
  SYNHANDLE SynHandle;	       /* $BGH7A=hM}%i%$%V%i%j%O%s%I%k(B */
  LNGHANDLE tLngHandle;	       /* $B8@8l=hM}%O%s%I%k(B */
  LNGOPEN tLngOpen;		/* $B8@8l=hM}%*!<%W%s9=B$BN(B */
  LNG tLng;				/* $B8@8l=hM}9=B$BN(B */
  pthread_mutex_t lock;
  dTalkParam();
  ~dTalkParam();
  int setSpeed(int);
  int setPunctuation(int);
};

struct ssWorkArg {
  dTalkParam *param;
  VoiceSet *vset;
  char src[SS_WORK_STR_MAX];
  int iLen;
  int type;
  int ret;
  int iSpeed;		/* $BH/@<%9%T!<%I(B */
  ssSharedInfo *info;
};

/* Function Prototypes */
int ss_close(int fd);
int open_audio(char *audevice);
#if SS_SYN_USE_ESD == TRUE
void ss_make_prog_name(char *namep, int len);
int open_audio(char *host, char *prog_name);
#endif
int sync_audio(int fd);
#if SS_SYN_USE_ESD == TRUE
void ss_reset_audio(char *prog_name, char *host);
#else
int ss_reset_audio(int fd);
#endif
int ssWarpPcm(short *data, int len, double factor);
char *ss_read_input(FILE *fp);
void ssInitWorkArg(ssWorkArg *wArg);
int dt_init(dTalkParam *dt);
int dt_close(dTalkParam *dt);
int dt_lang_anal(dTalkParam *dt, char *src);
void doBracketCommand(ssWorkArg *, const char *);
void *dt_speak(void *wArg); /* for pthread */
                          /* real argument type is (ssWorkArg *) */
void dt_stop(ssSharedInfo *info);

#ifdef DEBUG_OUTPUT
extern void log(char *);
#endif
#endif /* __SPCSRV_H__ */
