/*#define WRITE_ELOG*/
/*
** ssmain.cpp --- Main for LinuxSpeechServer.
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2001/03/27 15:52:42 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define DEBUG_OUTPUT 1
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <string>
#include "dic.h"
#include "spcsrv.h" /* speech server header file */
#include "ssqueue.h"
#include "cmdproc.h"

#ifdef WRITE_ELOG
void log_error(char *s, FILE *fpLog);
#endif

int main ( int iArgc, char **ppszArgv )
{
  char *read_buf;
  char *nextCmd;
  int iRet;
  dTalkParam dt;
  ss_queue sq;
  ssSharedInfo shInfo;
#ifdef WRITE_ELOG
  log_error("starting up\n", NULL);
#endif
  iRet = dt_init(&dt);
  if (iRet != SS_OPEN_OK) {
    printf("initialization failed\n");
    return 1;
  }

  printf("Bilingual Emacspeak Speech Server started!\n");

  while(1) {
    printf ("ready\n");
    read_buf = ss_read_input(stdin);
    if (read_buf == NULL) {
      printf("end of file on input stream\n");
      break;
    }
    printf("%s\n", read_buf);
#ifdef WRITE_ELOG
    log_error(read_buf, NULL);
#endif
    nextCmd = read_buf;
    while (*nextCmd == ' ') nextCmd++;
    if(*nextCmd == '\0') {
      printf("null command\n");
      continue;
    }

    if (*nextCmd == 's') {
      cmd_s(&sq, &shInfo, nextCmd, &nextCmd);
      continue;
    }
    else if (*nextCmd == 'q') {
      printf("queueing\n");
      iRet = cmd_q(&sq, nextCmd, &nextCmd);
      if (iRet != 0) {
	while (iRet != 0) {
	  printf ("queue continuing ready\n");
	  read_buf = ss_read_input(stdin);
	  if (read_buf == NULL) {
	    read_buf = "";
	  }
	  printf("%s\n", read_buf);
#ifdef WRITE_ELOG
	  log_error(read_buf, NULL);
#endif
	  nextCmd = read_buf;
	  iRet = cmd_q(&sq, nextCmd, &nextCmd);
	}
      }
    }
    else if (*nextCmd == 'd') {
      printf("speaking queue\n");
      cmd_d(&sq, &dt, &shInfo, nextCmd, &nextCmd);
    }
    else if (!strncmp(nextCmd, "tts_say", 7)) {
      printf("saying\n");
      cmd_tts_say(&sq, &dt, &shInfo, nextCmd, &nextCmd);
    }
    else if (*nextCmd == 'l') {
      printf("saying letter\n");
      cmd_tts_say(&sq, &dt, &shInfo, nextCmd, &nextCmd);
    }
    else if (!strncmp(nextCmd, "tts_set_speech_rate", 19)) {
      cmd_tts_set_speech_rate(sq.vset, nextCmd, &nextCmd);
    }
    else if (!strncmp(nextCmd, "tts_set_punctuations", 20)) {
      cmd_tts_set_punctuations(sq.vset, nextCmd, &nextCmd);
    }
    else {
      printf("ignoring\n");
      cmd_unknown(nextCmd, &nextCmd);
    }
  }
  iRet = dt_close(&dt);
  if (iRet != SS_CLOSE_OK) {
    printf("close failed\n");
    return 1;
  }
#ifdef WRITE_ELOG
  log_error("exited normally\n", NULL);
#endif
  return 0;
}

#ifdef WRITE_ELOG
void log_error(char *s, FILE *fpLog)
{
  FILE * fp;
  char FileName[256];
  if (fpLog == NULL) {
    sprintf(FileName, "%s/elog", getenv("HOME"));
    fp = fopen(FileName, "a");
    if (fp == NULL) printf("can't open log\n");
    else printf("opend %s\n", FileName);
  }
  else fp = fpLog;
  fprintf(fp, "%s\n", s);
  fflush(fp);
  if (fpLog == NULL) {
    fclose(fp);
  }
  return;
}
#endif

