/*
** ssqueue.cpp --- implementation of speech queue.
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2001/01/25 17:37:18 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <stdio.h>
#include <pthread.h>
#include <iostream>
#include <string>
#include "dic.h"
#include "spcsrv.h"
#include "ssqueue.h"

ss_queue_block::ss_queue_block() {
  next = prev = NULL;
}

ss_queue::ss_queue()
{
  top = NULL;
  end = NULL;
  vset = new VoiceSet();
}

ss_queue::~ss_queue()
{
  clear();
  delete vset;
}

ss_queue_block *
ss_queue::push_text(
		    const char *text,
		    int len
)
{
  ss_queue_block *qb;
  ss_queue_block *qtmp;

  if (top == NULL) {
    top = new ss_queue_block();
    if (top == NULL) {
      return NULL;
    }
    end = top;
    qb = top;
    qb->next = qb->prev = qb;
  } else {
    qb = end;
    qtmp = new ss_queue_block();
    if (qtmp == NULL) {
      return NULL;
    }
    qb->next = qtmp;
    qtmp->prev = qb;
    qtmp->next = qtmp;
    end = qtmp;
    qb = qtmp;
  }
  /* qb is always a new queue block */
  qb->text.assign(text, len);
  return qb;
}

ss_queue_block *
ss_queue::push_text_add(const char *text, int len)
{
  if (end == NULL) {
    return NULL;
  }
  end->text.append(text, len);
  return end;
}

void
ss_queue::pop(dTalkParam *dt, ssSharedInfo *info)
{
  ss_queue_block *qb;
  ssWorkArg *wArg = new ssWorkArg;
  pthread_t th;

  ssInitWorkArg(wArg);
  wArg->param = dt;
  wArg->vset = vset;
  wArg->info = info;

  if (empty()) {
    delete wArg;
    return;
  }
  qb = top;
  if (qb->text.length() >= SS_WORK_STR_MAX-1) {
    printf("too long line found:\n%s\n",
	   qb->text.c_str());
    strncpy(wArg->src, qb->text.c_str(), SS_WORK_STR_MAX-1);
    wArg->src[SS_WORK_STR_MAX-1] = '\0';
  } else {
    strcpy(wArg->src, qb->text.c_str());
    wArg->iLen = qb->text.length();
  }
  wArg->iSpeed = vset->iSpeed/dt->iSpeedFactor;
  if(wArg->iSpeed > DT_MAX_SPEED) {
    wArg->iSpeed = DT_MAX_SPEED;
  }
  wArg->type = SS_WORK_TYPE_PLAIN;
  pthread_create(&th, NULL, dt_speak, (void *)wArg);
  //pthread_detach(th);
  if (top->next == top) {
    delete qb;
    top = end = NULL;
  } else {
    top = qb -> next;
    delete qb;
  }
  return;
}

int
ss_queue::empty()
{
  if (top == NULL) {
    return 1;
  }
  return 0;
}

void
ss_queue::clear()
{
  ss_queue_block *qb;
  while (!empty()) {
    qb = top;
    top = qb->next;
    delete qb;
    if (top == qb) {
      top = end = NULL;
      break;
    }
  }
  return;
}


